# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmInit.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmInit(object):
    def setupUi(self, frmInit):
        frmInit.setObjectName("frmInit")
        frmInit.resize(437, 266)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(frmInit.sizePolicy().hasHeightForWidth())
        frmInit.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/configure.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmInit.setWindowIcon(icon)
        self.horizontalLayout = QtWidgets.QHBoxLayout(frmInit)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_4 = QtWidgets.QLabel(frmInit)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setScaledContents(False)
        self.label_4.setAlignment(QtCore.Qt.AlignCenter)
        self.label_4.setWordWrap(True)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(frmInit)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.cmbLanguage = QtWidgets.QComboBox(frmInit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbLanguage.sizePolicy().hasHeightForWidth())
        self.cmbLanguage.setSizePolicy(sizePolicy)
        self.cmbLanguage.setObjectName("cmbLanguage")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmbLanguage)
        self.lblServer = QtWidgets.QLabel(frmInit)
        self.lblServer.setObjectName("lblServer")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lblServer)
        self.txtServer = QtWidgets.QLineEdit(frmInit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtServer.sizePolicy().hasHeightForWidth())
        self.txtServer.setSizePolicy(sizePolicy)
        self.txtServer.setInputMethodHints(QtCore.Qt.ImhNone)
        self.txtServer.setEchoMode(QtWidgets.QLineEdit.Normal)
        self.txtServer.setObjectName("txtServer")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtServer)
        self.lblPort = QtWidgets.QLabel(frmInit)
        self.lblPort.setObjectName("lblPort")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.lblPort)
        self.txtPort = QtWidgets.QLineEdit(frmInit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtPort.sizePolicy().hasHeightForWidth())
        self.txtPort.setSizePolicy(sizePolicy)
        self.txtPort.setObjectName("txtPort")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtPort)
        self.lblUser = QtWidgets.QLabel(frmInit)
        self.lblUser.setObjectName("lblUser")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.lblUser)
        self.txtUser = QtWidgets.QLineEdit(frmInit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtUser.sizePolicy().hasHeightForWidth())
        self.txtUser.setSizePolicy(sizePolicy)
        self.txtUser.setObjectName("txtUser")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.txtUser)
        self.lblPass = QtWidgets.QLabel(frmInit)
        self.lblPass.setObjectName("lblPass")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.lblPass)
        self.txtPass = QtWidgets.QLineEdit(frmInit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtPass.sizePolicy().hasHeightForWidth())
        self.txtPass.setSizePolicy(sizePolicy)
        self.txtPass.setInputMethodHints(QtCore.Qt.ImhHiddenText|QtCore.Qt.ImhNoAutoUppercase|QtCore.Qt.ImhNoPredictiveText|QtCore.Qt.ImhSensitiveData)
        self.txtPass.setEchoMode(QtWidgets.QLineEdit.Password)
        self.txtPass.setObjectName("txtPass")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.txtPass)
        self.label_3 = QtWidgets.QLabel(frmInit)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.txtXulpymoney = QtWidgets.QLineEdit(frmInit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.txtXulpymoney.sizePolicy().hasHeightForWidth())
        self.txtXulpymoney.setSizePolicy(sizePolicy)
        self.txtXulpymoney.setObjectName("txtXulpymoney")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.txtXulpymoney)
        self.verticalLayout.addLayout(self.formLayout)
        self.cmdCreate = QtWidgets.QPushButton(frmInit)
        self.cmdCreate.setObjectName("cmdCreate")
        self.verticalLayout.addWidget(self.cmdCreate)
        self.line = QtWidgets.QFrame(frmInit)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(frmInit)
        QtCore.QMetaObject.connectSlotsByName(frmInit)
        frmInit.setTabOrder(self.txtPass, self.txtXulpymoney)
        frmInit.setTabOrder(self.txtXulpymoney, self.cmdCreate)
        frmInit.setTabOrder(self.cmdCreate, self.cmbLanguage)
        frmInit.setTabOrder(self.cmbLanguage, self.txtServer)
        frmInit.setTabOrder(self.txtServer, self.txtPort)
        frmInit.setTabOrder(self.txtPort, self.txtUser)

    def retranslateUi(self, frmInit):
        _translate = QtCore.QCoreApplication.translate
        frmInit.setWindowTitle(_translate("frmInit", "Initializating Xulpymoney database"))
        self.label_4.setText(_translate("frmInit", "<html><head/><body><p>Pressing the button below, you will create needed database to run Xulpymoney. It\'ll be created in the language selected</p></body></html>"))
        self.label.setText(_translate("frmInit", "Database language"))
        self.lblServer.setText(_translate("frmInit", "Server"))
        self.txtServer.setText(_translate("frmInit", "127.0.0.1"))
        self.lblPort.setText(_translate("frmInit", "Port"))
        self.txtPort.setText(_translate("frmInit", "5432"))
        self.lblUser.setText(_translate("frmInit", "User"))
        self.txtUser.setText(_translate("frmInit", "postgres"))
        self.lblPass.setText(_translate("frmInit", "Password"))
        self.label_3.setText(_translate("frmInit", "Xulpymoney Database"))
        self.txtXulpymoney.setText(_translate("frmInit", "xulpymoney"))
        self.cmdCreate.setText(_translate("frmInit", "Create database"))


import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgOpportunities.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgOpportunities(object):
    def setupUi(self, wdgOpportunities):
        wdgOpportunities.setObjectName("wdgOpportunities")
        wdgOpportunities.resize(824, 596)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/bank.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        wdgOpportunities.setWindowIcon(icon)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(wdgOpportunities)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lbl = QtWidgets.QLabel(wdgOpportunities)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout_2.addWidget(self.lbl)
        self.tabWidget = QtWidgets.QTabWidget(wdgOpportunities)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tab)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.wdgYear = wdgYear(self.tab)
        self.wdgYear.setObjectName("wdgYear")
        self.horizontalLayout.addWidget(self.wdgYear)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.cmbMode = QtWidgets.QComboBox(self.tab)
        self.cmbMode.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbMode.setObjectName("cmbMode")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/new.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbMode.addItem(icon1, "")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/expired.jpg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbMode.addItem(icon2, "")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbMode.addItem(icon3, "")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/eye.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmbMode.addItem(icon4, "")
        self.horizontalLayout_2.addWidget(self.cmbMode)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.tblOpportunities = myQTableWidget(self.tab)
        self.tblOpportunities.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblOpportunities.setAlternatingRowColors(True)
        self.tblOpportunities.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblOpportunities.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblOpportunities.setObjectName("tblOpportunities")
        self.tblOpportunities.setColumnCount(0)
        self.tblOpportunities.setRowCount(0)
        self.tblOpportunities.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tblOpportunities)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/editar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tab, icon5, "")
        self.verticalLayout_2.addWidget(self.tabWidget)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem4)
        self.label = QtWidgets.QLabel(wdgOpportunities)
        self.label.setObjectName("label")
        self.horizontalLayout_4.addWidget(self.label)
        self.txtInvest = myQLineEdit(wdgOpportunities)
        self.txtInvest.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtInvest.setObjectName("txtInvest")
        self.horizontalLayout_4.addWidget(self.txtInvest)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5.addLayout(self.verticalLayout_2)
        self.actionOpportunityNew = QtWidgets.QAction(wdgOpportunities)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpportunityNew.setIcon(icon6)
        self.actionOpportunityNew.setObjectName("actionOpportunityNew")
        self.actionOpportunityDelete = QtWidgets.QAction(wdgOpportunities)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/xulpymoney/button_cancel.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpportunityDelete.setIcon(icon7)
        self.actionOpportunityDelete.setObjectName("actionOpportunityDelete")
        self.actionOpportunityEdit = QtWidgets.QAction(wdgOpportunities)
        self.actionOpportunityEdit.setIcon(icon5)
        self.actionOpportunityEdit.setObjectName("actionOpportunityEdit")
        self.actionExecute = QtWidgets.QAction(wdgOpportunities)
        self.actionExecute.setIcon(icon3)
        self.actionExecute.setObjectName("actionExecute")
        self.actionRemove = QtWidgets.QAction(wdgOpportunities)
        self.actionRemove.setIcon(icon2)
        self.actionRemove.setObjectName("actionRemove")
        self.actionShowGraphic = QtWidgets.QAction(wdgOpportunities)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/xulpymoney/curiosity.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowGraphic.setIcon(icon8)
        self.actionShowGraphic.setObjectName("actionShowGraphic")

        self.retranslateUi(wdgOpportunities)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(wdgOpportunities)

    def retranslateUi(self, wdgOpportunities):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgOpportunities", "Opportunities list"))
        self.cmbMode.setItemText(0, _translate("wdgOpportunities", "Show current opportunities"))
        self.cmbMode.setItemText(1, _translate("wdgOpportunities", "Show removed opportunities"))
        self.cmbMode.setItemText(2, _translate("wdgOpportunities", "Show executed opportunities"))
        self.cmbMode.setItemText(3, _translate("wdgOpportunities", "Show all"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("wdgOpportunities", "Opportunity annotations list"))
        self.label.setText(_translate("wdgOpportunities", "Amount to invest in opportunity product currency"))
        self.actionOpportunityNew.setText(_translate("wdgOpportunities", "New purchase opportunity"))
        self.actionOpportunityNew.setToolTip(_translate("wdgOpportunities", "New purchase opportunity"))
        self.actionOpportunityDelete.setText(_translate("wdgOpportunities", "Delete purchase opportunity"))
        self.actionOpportunityDelete.setToolTip(_translate("wdgOpportunities", "Delete purchase opportunity"))
        self.actionOpportunityEdit.setText(_translate("wdgOpportunities", "Edit purchase opportunity"))
        self.actionOpportunityEdit.setToolTip(_translate("wdgOpportunities", "Edit purchase opportunity"))
        self.actionExecute.setText(_translate("wdgOpportunities", "Execute opportunity"))
        self.actionExecute.setToolTip(_translate("wdgOpportunities", "Execute opportunity"))
        self.actionRemove.setText(_translate("wdgOpportunities", "Remove opportunity"))
        self.actionRemove.setToolTip(_translate("wdgOpportunities", "Remove opportunity"))
        self.actionShowGraphic.setText(_translate("wdgOpportunities", "Show opportunity graphic"))
        self.actionShowGraphic.setToolTip(_translate("wdgOpportunities", "Show opportunity graphic"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.ui.wdgYear import wdgYear
import xulpymoney.images.xulpymoney_rc



import datetime
import logging
import pytz
from PyQt5.QtCore import Qt,  pyqtSlot
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QDialog,  QMenu, QMessageBox,  QFileDialog
from xulpymoney.ui.Ui_frmProductReport import Ui_frmProductReport
from xulpymoney.libxulpymoney import DPS, Percentage, Product, Quote, AgrupationManager, QuoteManager, QuoteAllIntradayManager, StockMarketManager,  CurrencyManager, LeverageManager, ProductModesManager, ProductTypeManager
from xulpymoney.libxulpymoneyfunctions import c2b, day_end, dtaware, qcenter, qdatetime, qleft, dtaware2string, qmessagebox
from xulpymoney.libxulpymoneytypes import eDtStrings
from xulpymoney.version import __version__, __versiondate__
from xulpymoney.ui.frmSelector import frmSelector
from xulpymoney.ui.frmDividendsAdd import frmDividendsAdd
from xulpymoney.ui.frmQuotesIBM import frmQuotesIBM
from xulpymoney.ui.frmSplit import frmSplit
from xulpymoney.ui.frmSplitManual import frmSplitManual
from xulpymoney.ui.frmEstimationsAdd import frmEstimationsAdd
from xulpymoney.ui.frmDPSAdd import frmDPSAdd
from xulpymoney.ui.wdgProductHistoricalChart import wdgProductHistoricalChart
from xulpymoney.ui.canvaschart import  VCTemporalSeries
from officegenerator import ODS_Read, ODS_Write, Currency as ODSCurrency, Coord, ColumnWidthODS

class frmProductReport(QDialog, Ui_frmProductReport):
    def __init__(self, mem,  product, inversion=None, parent = None, name = None, modal = False):
        """
            self.product.id==None Insertar
            self.product.id <0 Editar
            self.product.id >0 Red ONly
        """
        QDialog.__init__(self,  parent)
        self.hide()
        QApplication.setOverrideCursor(Qt.WaitCursor)
        self.setupUi(self)

        self.mem=mem
        self.product=product
        self.investment=inversion#Used to generate puntos de venta, punto de compra....
        self.setSelIntraday=set([])
        
        self.adding_new_product=False#Tag to know is I access this dialog adding a product
        self.selDPS=None
        self.selEstimationDPS=None
        self.selEstimationEPS=None
        
        self.selDate=None #Fecha seleccionado en datos historicos
        self.selDateTime=None #Datetime seleccionado para borrar quote no es el mismo procedimiento de borrado
        self.tab.setCurrentIndex(1)
        self.tabGraphics.setCurrentIndex(1)
        self.tabHistorical.setCurrentIndex(4)
        
        self.tblTPC.settings(self.mem, "frmProductReport")
        self.tblDaily.settings(self.mem, "frmProductReport")    
        self.tblWeekly.settings(self.mem, "frmProductReport")
        self.tblMonthly.settings(self.mem, "frmProductReport")    
        self.tblYearly.settings(self.mem, "frmProductReport")    
        self.tblIntradia.settings(self.mem, "frmProductReport")    
        self.tblMensuales.settings(self.mem, "frmProductReport")    
        self.tblSplits.settings(self.mem, "frmProductReport")
        self.tblDividendsEstimations.settings(self.mem, "frmProductReport")    
        self.tblDPSPaid.settings(self.mem, "frmProductReport")
        self.tblEPS.settings(self.mem, "frmProductReport")


        if self.product==None: #Insertar
            self.adding_new_product=True
            self.product=Product(self.mem)
            self.cmdSave.setText(self.tr("Add a new product"))
            
            self.tab.setTabEnabled(1, False)
            self.tab.setTabEnabled(2, False)
            self.tab.setTabEnabled(3, False)
            self.mem.stockmarkets.qcombobox(self.cmbBolsa)
            self.mem.investmentsmodes.qcombobox(self.cmbPCI)
            self.mem.currencies.qcombobox(self.cmbCurrency)
            self.mem.leverages.qcombobox(self.cmbApalancado)
            self.mem.types.qcombobox(self.cmbTipo)
        elif self.product.id<0: #Editar
            self.mem.stockmarkets.qcombobox(self.cmbBolsa)
            self.mem.investmentsmodes.qcombobox(self.cmbPCI)
            self.mem.currencies.qcombobox(self.cmbCurrency)
            self.mem.leverages.qcombobox(self.cmbApalancado)
            self.mem.types.qcombobox(self.cmbTipo)
        elif self.product.id>=0:#Readonly
            self.txtISIN.setReadOnly(True)
            self.txtName.setReadOnly(True)
            self.txtWeb.setReadOnly(True)
            self.txtAddress.setReadOnly(True)
            self.txtMail.setReadOnly(True)
            self.txtTPC.setReadOnly(True)
            self.txtPhone.setReadOnly(True)
            self.tblTickers.setEnabled(False)
            self.txtComentario.setReadOnly(True)
            self.cmdAgrupations.setEnabled(False)
            self.chkObsolete.setEnabled(False)
            self.chkHL.setEnabled(False)
            self.cmdSave.setEnabled(False)
            
            bolsa=StockMarketManager(mem)
            bolsa.append(self.product.stockmarket)
            bolsa.qcombobox(self.cmbBolsa)
            
            productmodes=ProductModesManager(mem)
            productmodes.append(self.product.mode)
            productmodes.qcombobox(self.cmbPCI)
            
            currencies=CurrencyManager(mem)
            currencies.append(self.product.currency)
            currencies.qcombobox(self.cmbCurrency)
            
            leverages=LeverageManager(mem)
            leverages.append(self.product.leveraged)
            leverages.qcombobox(self.cmbApalancado)
            
            types=ProductTypeManager(mem)
            types.append(self.product.type)
            types.qcombobox(self.cmbTipo)
        
        self.viewIntraday=VCTemporalSeries()
        self.layIntraday.addWidget(self.viewIntraday)
       
        self.wdgHistorical=wdgProductHistoricalChart(self)
        self.layHistorical.addWidget(self.wdgHistorical)

        self.update_due_to_quotes_change()    
        self.showMaximized()        
        QApplication.restoreOverrideCursor()
        
        
    def on_tabHistorical_currentChanged(self, index):
        def setTable(table, data):
            table.setRowCount(len(data.arr))
            table.applySettings()
            table.clearContents()
            if len(data.arr)==0:
                return
            for punt, d in enumerate(data.arr):
                table.setItem(punt, 0, qcenter(d.print_time())) 
                table.setItem(punt, 1, self.product.currency.qtablewidgetitem(d.close,6))
                table.setItem(punt, 2, self.product.currency.qtablewidgetitem(d.open,6))
                table.setItem(punt, 3, self.product.currency.qtablewidgetitem(d.high,6))
                table.setItem(punt, 4, self.product.currency.qtablewidgetitem(d.low,6))
                table.setItem(punt, 5, qcenter(str(d.datetime())))
            table.setCurrentCell(len(data.arr)-1, 0)
            table.setFocus()
        ## load_historicas
        if index==0:
            setTable(self.tblDaily, self.product.result.ohclDailyBeforeSplits)
        elif index==1:
            setTable(self.tblWeekly, self.product.result.ohclWeekly)
        elif index==2:
            setTable(self.tblMonthly, self.product.result.ohclMonthly)
        elif index==3:
            setTable(self.tblYearly, self.product.result.ohclYearly)

        
    def load_information(self):
        def row_tblTPV(quote,  row):
            if quote==None:
                return
            self.tblTPC.setItem(row, 0, qdatetime(quote.datetime, self.mem.localzone))
            self.tblTPC.setItem(row, 1, self.product.currency.qtablewidgetitem(quote.quote, 6))

            try:
                tpc=Percentage(self.product.result.basic.last.quote-quote.quote, quote.quote)
                days=(datetime.datetime.now(pytz.timezone(self.mem.localzone.name))-quote.datetime).days+1
                self.tblTPC.setItem(row, 2, tpc.qtablewidgetitem())
                self.tblTPC.setItem(row, 3,  (tpc*365/days).qtablewidgetitem())
                if self.investment:
                    self.grpHistoricos.setTitle(self.tr('Report of historic prices. You have {} shares valued at {}.').format(self.investment.shares(), self.investment.balance()))
                    self.tblTPC.setItem(row, 4,  self.product.currency.qtablewidgetitem(self.investment.shares()*(self.product.result.basic.last.quote-quote.quote)))
            except:
                self.tblTPC.setItem(row, 2, Percentage().qtablewidgetitem())    
                self.tblTPC.setItem(row, 3,  Percentage().qtablewidgetitem())
                self.tblTPC.setItem(row, 3,  self.product.currency.qtablewidgetitem(None))     

        self.product.agrupations.qcombobox(self.cmbAgrupations)

        self.lblInvestment.setText("{} ( {} )".format(self.product.name, self.product.id))
        self.txtTPC.setText(str(self.product.percentage))
        self.txtName.setText(self.product.name)
        self.txtISIN.setText(self.product.isin)
        for i, ticker in enumerate(self.product.tickers):
            if ticker==None:
                self.tblTickers.setItem(i, 0, qleft(""))
            else:
                self.tblTickers.setItem(i, 0, qleft(self.product.tickers[i]))
        self.txtComentario.setText(self.product.comment)
        self.txtAddress.setText(self.product.address)
        self.txtWeb.setText(self.product.web)
        self.txtMail.setText(self.product.mail)
        self.txtPhone.setText(self.product.phone)

        if self.product.has_autoupdate()==True:
            self.lblAutoupdate.setText('<img src=":/xulpymoney/transfer.png" width="16" height="16"/>  {}'.format(self.tr("Product prices are updated automatically")))
        else:
            self.lblAutoupdate.setText(self.tr("Product prices are not updated automatically"))
            
        if self.product.obsolete==True:
            self.chkObsolete.setCheckState(Qt.Checked)
        
        if self.product.high_low==True:
            self.chkHL.setCheckState(Qt.Checked)

        self.cmbBolsa.setCurrentIndex(self.cmbBolsa.findData(self.product.stockmarket.id))
        self.cmbCurrency.setCurrentIndex(self.cmbCurrency.findData(self.product.currency.id))
        self.cmbPCI.setCurrentIndex(self.cmbPCI.findData(self.product.mode.id))
        self.cmbTipo.setCurrentIndex(self.cmbTipo.findData(self.product.type.id))
        self.cmbApalancado.setCurrentIndex(self.cmbApalancado.findData(self.product.leveraged.id))
        
        if len(self.product.result.ohclDaily.arr)!=0:
            now=self.mem.localzone.now()
            penultimate=self.product.result.basic.penultimate
            iniciosemana=Quote(self.mem).init__from_query(self.product,  day_end(now-datetime.timedelta(days=datetime.date.today().weekday()+1), self.product.stockmarket.zone))
            iniciomes=Quote(self.mem).init__from_query(self.product, dtaware(datetime.date(now.year, now.month, 1), datetime.time(0, 0), self.product.stockmarket.zone.name))
            inicioano=Quote(self.mem).init__from_query(self.product, dtaware(datetime.date(now.year, 1, 1), datetime.time(0, 0), self.product.stockmarket.zone.name))             
            docemeses=Quote(self.mem).init__from_query(self.product, day_end(now-datetime.timedelta(days=365), self.product.stockmarket.zone))          
            unmes=Quote(self.mem).init__from_query(self.product, day_end(now-datetime.timedelta(days=30), self.product.stockmarket.zone))          
            unasemana=Quote(self.mem).init__from_query(self.product, day_end(now-datetime.timedelta(days=7), self.product.stockmarket.zone))             
            
            self.tblTPC.applySettings()
            self.tblTPC.setItem(0, 0, qdatetime(self.product.result.basic.last.datetime, self.mem.localzone))   
            self.tblTPC.setItem(0, 1, self.product.currency.qtablewidgetitem(self.product.result.basic.last.quote,  6))
            
            row_tblTPV(penultimate, 2)
            row_tblTPV(iniciosemana, 3)## Para que sea el domingo
            row_tblTPV(iniciomes, 4)
            row_tblTPV(inicioano, 5)
            row_tblTPV(unasemana, 7)
            row_tblTPV(unmes, 8)
            row_tblTPV(docemeses, 9)

        
    def update_due_to_quotes_change(self):
        if self.product.id!=None:
            self.product.needStatus(2)
            self.product.result.ohclDaily.selected=[]
            self.product.result.ohclMonthly.selected=[]
            self.product.result.ohclWeekly.selected=[]
            self.product.result.ohclYearly.selected=[]
            self.product.estimations_dps.load_from_db()#No cargada por defecto en product
            self.product.estimations_eps.load_from_db()#No cargada por defecto en product

            self.product.estimations_dps.myqtablewidget(self.tblDividendsEstimations)   
            self.product.estimations_eps.myqtablewidget(self.tblEPS)            
            self.product.dps.myqtablewidget(self.tblDPSPaid)         
            self.product.splits.myqtablewidget(self.tblSplits)
            inicio=datetime.datetime.now()
            self.load_information()
            logging.info("Datos informacion cargados: {}".format(datetime.datetime.now()-inicio))
            self.load_graphics()
            logging.info("Datos gráficos cargados: {}".format(datetime.datetime.now()-inicio))
            self.load_mensuales()
            logging.info("Datos mensuales cargados: {}".format(datetime.datetime.now()-inicio))
            self.on_tabHistorical_currentChanged(self.tabHistorical.currentIndex())


        

    def load_graphics(self):
        self.product.result.get_intraday(self.calendar.selectedDate().toPyDate())
        
        #Canvas Historical
        if len(self.product.result.ohclDaily.arr)<2:#Needs 2 to show just a line
            pass
        else:
            self.wdgHistorical.setProduct(self.product, self.investment)
            self.wdgHistorical.generate()
            self.wdgHistorical.display()

        #Canvas Intradia
        if self.product.result.intradia.length()<2:
            self.viewIntraday.hide()
        else:
            self.viewIntraday.show()
            self.layIntraday.removeWidget(self.viewIntraday)
            self.viewIntraday.close()
            
            self.viewIntraday=VCTemporalSeries()
            self.layIntraday.addWidget(self.viewIntraday)
            self.viewIntraday.chart.setTitle(self.tr("Intraday graph"))
            ls=self.viewIntraday.appendTemporalSeries(self.product.name.upper(), self.product.currency)
            for quote in self.product.result.intradia.arr:
                self.viewIntraday.appendTemporalSeriesData(ls, quote.datetime, quote.quote)
            self.viewIntraday.display()

        #tblIntradia
        try:
            self.product.result.intradia.myqtablewidget(self.tblIntradia)
            if self.product.result.intradia.length()>0:
                self.lblIntradayVariance.setText(self.tr("Daily maximum variance: {} ({})").format(self.product.currency.string(self.product.result.intradia.variance()), self.product.result.intradia.variance_percentage()))
        except:
            logging.error("Error creating intraday table. Perhaps due to currency exchange missing quotes")

    def load_mensuales(self):
        if len(self.product.result.ohclMonthly.arr)==0:
            self.tblMensuales.clear()
            return

        minyear=self.product.result.ohclMonthly.arr[0].year
        rowcount=int(datetime.date.today().year-minyear+1)
        self.tblMensuales.applySettings()
        self.tblMensuales.setRowCount(rowcount)    

        for i, year in enumerate(range(minyear,  datetime.date.today().year+1)):
            self.tblMensuales.setItem(i, 0, qleft(year))
            for month in range(1, 13):
                self.tblMensuales.setItem(i, month, self.product.result.ohclMonthly.percentage_by_year_month(year, month).qtablewidgetitem())
            self.tblMensuales.setItem(i, 13, self.product.result.ohclYearly.percentage_by_year(year).qtablewidgetitem())

    @pyqtSlot() 
    def on_actionDividendXuNew_triggered(self):
        w=frmDividendsAdd(self.mem, self.investment,  None)
        w.wdgDT.setCombine(self.mem, self.selDPS.date, self.product.stockmarket.closes, self.product.stockmarket.zone)
        gross=self.selDPS.gross*self.investment.shares(self.selDPS.date)
        w.txtBruto.setText(gross)
        w.txtDPA.setText(self.selDPS.gross)
        w.txtRetencion.setText(gross*self.mem.taxcapitalappreciation)
        w.cmb.setCurrentIndex(w.cmb.findData(39))
        w.exec_()

    @pyqtSlot()
    def on_actionDPSDelete_triggered(self):
        if self.selDPS!=None:
            for i in self.tblDPSPaid.selectedItems():#itera por cada item no row.        
                if i.column()==0:
                    dps=self.product.dps.arr[i.row()]
                    dps.borrar()
            self.mem.con.commit()
            self.product.result.load_dps_and_splits(force=True)#Reload becouse, deleting one in one loose index reference
            self.product.needStatus(2, downgrade_to=1)
            self.update_due_to_quotes_change()

        
    @pyqtSlot()
    def on_actionDPSNew_triggered(self):
        d=frmDPSAdd(self.mem, self.product)
        d.exec_()
        self.product.dps.myqtablewidget(self.tblDPSPaid)
        self.product.needStatus(2, downgrade_to=1)
        self.update_due_to_quotes_change()
        
    @pyqtSlot()
    def on_actionDPSImport_triggered(self):
        filename=QFileDialog.getOpenFileName(self, "", "", "LibreOffice Calc (*.ods)")[0]
        got=0
        if filename!="":
            ods=ODS_Read(filename)
            sheet=ods.getSheetElementByIndex(0)
            for number in range(2, ods.rowNumber(sheet)):
                date=ods.getCellValue(sheet, "A", str(number))
                value=ods.getCellValue(sheet, "B", str(number))
                print (date, value)
                dps=DPS(self.mem, self.product).init__create(date,  value)
                dps.save()
                self.product.dps.append(dps)
                got=got+1
            print("Added {} DPS from {} ODS rows".format(got, ods.rowNumber(sheet)-1))
            self.mem.con.commit()
            self.product.needStatus(2, downgrade_to=1)
            self.update_due_to_quotes_change()

    @pyqtSlot()
    def on_actionEstimationDPSDelete_triggered(self):
        if self.selEstimationDPS!=None:
            self.selEstimationDPS.borrar()
            self.product.estimations_dps.arr.remove(self.selEstimationDPS)
            self.mem.con.commit()
            self.product.estimations_dps.myqtablewidget(self.tblDividendsEstimations)
            self.product.needStatus(2, downgrade_to=0)
        
    @pyqtSlot()
    def on_actionEstimationDPSNew_triggered(self):
        d=frmEstimationsAdd(self.mem, self.product, "dps")
        d.exec_()
        self.product.estimations_dps.myqtablewidget(self.tblDividendsEstimations)

    @pyqtSlot()
    def on_actionEstimationEPSDelete_triggered(self):
        if self.selEstimationEPS!=None:
            self.selEstimationEPS.borrar()
            self.product.estimations_eps.arr.remove(self.selEstimationEPS)
            self.mem.con.commit()
            self.product.estimations_eps.myqtablewidget(self.tblEPS)
            self.product.needStatus(2, downgrade_to=0)
        
    @pyqtSlot()
    def on_actionEstimationEPSNew_triggered(self):
        d=frmEstimationsAdd(self.mem, self.product, "eps")
        d.exec_()
        self.product.estimations_eps.myqtablewidget(self.tblEPS)

    @pyqtSlot()
    def on_actionPurgeDay_triggered(self):
        self.product.result.intradia.purge()
        self.mem.con.commit()
        self.load_graphics()#OHLC ya estaba cargado, no varía por lo que no uso update_due_to_quotes_change
        
    @pyqtSlot()
    def on_actionQuoteEdit_triggered(self):
        for quote in self.setSelIntraday:##Only is one, but i don't know how to refer to quote
            w=frmQuotesIBM(self.mem,  self.product, quote)
            w.exec_()   
            if w.result()==QDialog.Accepted:
                self.product.needStatus(2, downgrade_to=0)
                self.update_due_to_quotes_change()
        
    @pyqtSlot()
    def on_actionQuoteExport_triggered(self):
        start=datetime.datetime.now()
        filename_ods="{} Quotes of {}.ods".format(dtaware2string(self.mem.localzone.now(), type=eDtStrings.Filename),  self.product.name)    
        filename = QFileDialog.getSaveFileName(self, self.tr("Save File"), filename_ods, self.tr("Libreoffice calc (*.ods)"))[0]
        if filename:
            ods=ODS_Write(filename)
            ods.setMetadata(self.tr("Historical quotes of {}").format(self.product.name),  self.tr("Quotes export"), "Xulpymoney-{} ({})".format(__version__, __versiondate__))
            s1=ods.createSheet(self.tr("Intraday"))
            s1.add("A1", [["Date and time", "Close", "Open","High","Low"]], "OrangeCenter")
            s1.setColumnsWidth([ColumnWidthODS.L])
            for i, o in enumerate(self.product.result.ohclDaily.arr):
                s1.add(Coord("A2").addRow(i), o.date, "WhiteDate")
                s1.add(Coord("B2").addRow(i), ODSCurrency(o.close, self.product.currency.id), "WhiteEUR")
                s1.add(Coord("C2").addRow(i), ODSCurrency(o.open, self.product.currency.id), "WhiteEUR")
                s1.add(Coord("D2").addRow(i), ODSCurrency(o.high, self.product.currency.id), "WhiteEUR")
                s1.add(Coord("E2").addRow(i), ODSCurrency(o.low, self.product.currency.id), "WhiteEUR")
            s1.setCursorPosition(Coord("A1").addRow(self.product.result.ohclDaily.length()))
            s1.setSplitPosition("A2")
            ods.save()
            qmessagebox(self.tr("Date export to {} took {}").format(filename, datetime.datetime.now()-start))
 
        
    @pyqtSlot()
    def on_actionQuoteImport_triggered(self):
        filename=QFileDialog.getOpenFileName(self, "", "", "LibreOffice Calc (*.ods)")[0]
        got=0
        if filename!="":
            set=QuoteManager(self.mem)
            ods=ODS_Read(filename)
            sheet=ods.getSheetElementByIndex(0)
            for number in range(2, ods.rowNumber(sheet)):
                date=ods.getCellValue(sheet, "A"+ str(number))
                value=ods.getCellValue(sheet, "B"+ str(number))
                print (date, value)
                set.append(Quote(self.mem).init__create(self.product, dtaware(date, self.product.stockmarket.closes, self.product.stockmarket.zone.name),  value))
                got=got+1
            print("Added {} DPS from {} ODS rows".format(got, ods.rowNumber(sheet)-1))
            set.save()
            self.mem.con.commit()
            self.product.needStatus(2, downgrade_to=0)
            self.update_due_to_quotes_change()

    @pyqtSlot()
    def on_actionQuoteNew_triggered(self):
        w=frmQuotesIBM(self.mem,  self.product)
        w.wdgDT.teDate.setSelectedDate(self.calendar.selectedDate())
        w.exec_()   
        if w.result()==QDialog.Accepted:
            self.product.needStatus(2, downgrade_to=0)
            self.update_due_to_quotes_change()

    @pyqtSlot()
    def on_actionQuoteDelete_triggered(self):
        for q in self.setSelIntraday:
            q.delete()
            self.product.result.intradia.arr.remove(q)
        self.mem.con.commit()
        self.product.needStatus(2, downgrade_to=0)
        self.update_due_to_quotes_change()
        
    @pyqtSlot()
    def on_actionQuoteDeleteDays_triggered(self):
        for ohcl in self.product.result.ohclDaily.selected:
            ohcl.delete()
        self.mem.con.commit()
        self.product.needStatus(2, downgrade_to=0)
        self.update_due_to_quotes_change()
    
    @pyqtSlot()
    def on_actionQuoteDeleteMonths_triggered(self):
        for ohcl in self.product.result.ohclMonthly.selected:
            ohcl.delete()
        self.mem.con.commit()
        self.product.needStatus(2, downgrade_to=0)
        self.update_due_to_quotes_change()
    
    @pyqtSlot()
    def on_actionQuoteDeleteYears_triggered(self):
        for ohcl in self.product.result.ohclYearly.selected:
            ohcl.delete()
        self.mem.con.commit()
        self.product.needStatus(2, downgrade_to=0)
        self.update_due_to_quotes_change()

    def on_calendar_selectionChanged(self):
        self.load_graphics()

    def on_cmdSplitManual_pressed(self):
        w=frmSplitManual(self.mem, self.product)
        w.exec_()   
        if w.result()==QDialog.Accepted:
            self.product.needStatus(2, downgrade_to=0)
            self.update_due_to_quotes_change()
            
    @pyqtSlot()
    def on_actionSplitNew_triggered(self):
        w=frmSplit(self.mem, self.product)
        w.exec_()   
        if w.result()==QDialog.Accepted:
            self.mem.con.commit()
            self.product.splits.append(w.split)
            self.product.splits.order_by_datetime()
            self.product.needStatus(2, downgrade_to=0)
            self.update_due_to_quotes_change()
    
    
    @pyqtSlot()
    def on_actionSplitEdit_triggered(self):
        w=frmSplit(self.mem, self.product, self.product.splits.selected)
        w.exec_()   
        if w.result()==QDialog.Accepted:
            self.product.needStatus(2, downgrade_to=0)
            self.update_due_to_quotes_change()

    @pyqtSlot()
    def on_actionSplitRemove_triggered(self):
        self.product.splits.delete(self.product.splits.selected)
        self.mem.con.commit()
        self.product.needStatus(2, downgrade_to=0)
        self.update_due_to_quotes_change()
        
    def on_cmdPurge_pressed(self):
        all=QuoteAllIntradayManager(self.mem)
        all.load_from_db(self.product)
        numpurged=all.purge(progress=True)
        if numpurged!=None:#Canceled
            self.mem.con.commit()
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setIcon(QMessageBox.Information)
            m.setText(self.tr("{0} quotes have been purged from {1}".format(numpurged, self.product.name)))
            m.exec_()    
        else:
            self.mem.con.rollback()
        
    def on_cmdSave_pressed(self):
        if self.product.id==None or self.product.id<0:
            self.product.name=self.txtName.text()
            if self.txtISIN.text()=="":
                self.product.isin=None
            else:
                self.product.isin=self.txtISIN.text()
            self.product.currency=self.mem.currencies.find_by_id(self.cmbCurrency.itemData(self.cmbCurrency.currentIndex()))
            self.product.type=self.mem.types.find_by_id(self.cmbTipo.itemData(self.cmbTipo.currentIndex()))
            self.product.agrupations=AgrupationManager(self.mem).clone_from_combo(self.cmbAgrupations)
            self.product.obsolete=c2b(self.chkObsolete.checkState())
            self.product.high_low=c2b(self.chkHL.checkState())
            self.product.web=self.txtWeb.text()
            self.product.address=self.txtAddress.text()
            self.product.phone=self.txtPhone.text()
            self.product.mail=self.txtMail.text()
            self.product.percentage=int(self.txtTPC.text())
            self.product.mode=self.mem.investmentsmodes.find_by_id(self.cmbPCI.itemData(self.cmbPCI.currentIndex()))
            self.product.leveraged=self.mem.leverages.find_by_id(self.cmbApalancado.itemData(self.cmbApalancado.currentIndex()))
            self.product.stockmarket=self.mem.stockmarkets.find_by_id(self.cmbBolsa.itemData(self.cmbBolsa.currentIndex()))
            for i in range(self.tblTickers.rowCount()):
                value=self.tblTickers.item(i, 0).text()
                if value =="":
                    value=None
                self.product.tickers[i]=value
            self.product.comment=self.txtComentario.text()                
            self.product.save()
            self.mem.con.commit()  
            self.mem.data.products.append(self.product)
            self.done(0)
        elif self.product.id>0:
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setText("Only developers can change system products. You can create a new personal product or fill a ticket in the sourceforge page. It will be updated soon")
            m.setIcon(QMessageBox.Information)
            m.exec_()        
            return

    def on_cmdAgrupations_released(self):
        ##Se debe clonar, porque selector borra
        if self.cmbTipo.itemData(self.cmbTipo.currentIndex())==2:#Fondos de inversión
            agr=self.mem.agrupations.clone_fondos()
        elif self.cmbTipo.itemData(self.cmbTipo.currentIndex())==1:#Acciones
            agr=self.mem.agrupations.clone_acciones()
        elif self.cmbTipo.itemData(self.cmbTipo.currentIndex())==4:#ETFs
            agr=self.mem.agrupations.clone_etfs()
        elif self.cmbTipo.itemData(self.cmbTipo.currentIndex())==5:#Warrants
            agr=self.mem.agrupations.clone_warrants()
        else:
            agr=self.mem.agrupations.clone(self.mem)
        if self.product.agrupations==None:
            selected=AgrupationManager(self.mem)#Vacio
        else:
            selected=self.product.agrupations
        f=frmSelector(self.mem, agr, selected)
        f.lbl.setText(self.tr("Agrupation selection"))
        f.exec_()
        f.selected.qcombobox(self.cmbAgrupations)

    def on_tblDaily_itemSelectionChanged(self):
        if self.product.result.ohclDaily.selected!=None:
            del self.product.result.ohclDaily.selected
            self.product.result.ohclDaily.selected=[]
            
        for i in self.tblDaily.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.product.result.ohclDaily.selected.append(self.product.result.ohclDaily.arr[i.row()])

    def on_tblDaily_customContextMenuRequested(self,  pos):
        if len(self.product.result.ohclDaily.selected)>0:
            self.actionQuoteDeleteDays.setEnabled(True)
        else:
            self.actionQuoteDeleteDays.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionQuoteDeleteDays)        
        menu.addSeparator()
        menu.addAction(self.actionQuoteImport)
        menu.addAction(self.actionQuoteExport)
        menu.exec_(self.tblDaily.mapToGlobal(pos))
        
    def on_tblMonthly_itemSelectionChanged(self):
        if self.product.result.ohclMonthly.selected!=None:
            del self.product.result.ohclMonthly.selected
            self.product.result.ohclMonthly.selected=[]
            
        for i in self.tblMonthly.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.product.result.ohclMonthly.selected.append(self.product.result.ohclMonthly.arr[i.row()])

    def on_tblMonthly_customContextMenuRequested(self,  pos):
        if len(self.product.result.ohclMonthly.selected)>0:
            self.actionQuoteDeleteMonths.setEnabled(True)
        else:
            self.actionQuoteDeleteMonths.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionQuoteDeleteMonths)        
        menu.exec_(self.tblMonthly.mapToGlobal(pos))  
        
    def on_tblYearly_itemSelectionChanged(self):
        if self.product.result.ohclYearly.selected!=None:
            del self.product.result.ohclYearly.selected
            self.product.result.ohclYearly.selected=[]
            
        for i in self.tblYearly.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.product.result.ohclYearly.selected.append(self.product.result.ohclYearly.arr[i.row()])

    def on_tblYearly_customContextMenuRequested(self,  pos):
        if len(self.product.result.ohclYearly.selected)>0:
            self.actionQuoteDeleteYears.setEnabled(True)
        else:
            self.actionQuoteDeleteYears.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionQuoteDeleteYears)        
        menu.exec_(self.tblYearly.mapToGlobal(pos))
    def on_tblIntradia_customContextMenuRequested(self,  pos):
        if len (self.setSelIntraday)>0:
            self.actionQuoteDelete.setEnabled(True)
        else:
            self.actionQuoteDelete.setEnabled(False)

        if len(self.setSelIntraday)==1:
            self.actionQuoteEdit.setEnabled(True)
        else:
            self.actionQuoteEdit.setEnabled(False)
            
        menu=QMenu()
        menu.addAction(self.actionQuoteNew)
        menu.addAction(self.actionQuoteEdit)
        menu.addAction(self.actionQuoteDelete)        
        menu.addSeparator()
        menu.addAction(self.actionPurgeDay)
        menu.exec_(self.tblIntradia.mapToGlobal(pos))

    def on_tblIntradia_itemSelectionChanged(self):
        sel=[]
        try:
            for i in self.tblIntradia.selectedItems():#itera por cada item no row.
                if i.column()==0:
                    sel.append(self.product.result.intradia.arr[i.row()])
            self.setSelIntraday=set(sel)
        except:
            self.setSelIntraday=set([])
            
            
    def on_tblDividendsEstimations_itemSelectionChanged(self):
        try:
            for i in self.tblDividendsEstimations.selectedItems():#itera por cada item no row.        
                if i.column()==0:
                    self.selEstimationDPS=self.product.estimations_dps.arr[i.row()]
        except:
            self.selEstimationDPS=None
            
    def on_tblDividendsEstimations_customContextMenuRequested(self,  pos):
        if self.selEstimationDPS==None:
            self.actionEstimationDPSDelete.setEnabled(False)
        else:
            self.actionEstimationDPSDelete.setEnabled(True)
        menu=QMenu()
        menu.addAction(self.actionEstimationDPSNew)
        menu.addAction(self.actionEstimationDPSDelete)    
        menu.exec_(self.tblDividendsEstimations.mapToGlobal(pos))
            
            
    def on_tblEPS_itemSelectionChanged(self):
        try:
            for i in self.tblEPS.selectedItems():#itera por cada item no row.        
                if i.column()==0:
                    self.selEstimationEPS=self.product.estimations_eps.arr[i.row()]
        except:
            self.selEstimationEPS=None
            
    def on_tblSplits_customContextMenuRequested(self,  pos):
        if self.product.splits.selected==None:
            self.actionSplitEdit.setEnabled(False)
            self.actionSplitRemove.setEnabled(False)
        else:
            self.actionSplitEdit.setEnabled(True)
            self.actionSplitRemove.setEnabled(True)
        menu=QMenu()
        menu.addAction(self.actionSplitNew)
        menu.addAction(self.actionSplitEdit)
        menu.addAction(self.actionSplitRemove)
        menu.exec_(self.tblSplits.mapToGlobal(pos))
            
    def on_tblSplits_itemSelectionChanged(self):
        try:
            for i in self.tblSplits.selectedItems():#itera por cada item no row.        
                if i.column()==0:
                    self.product.splits.selected=self.product.splits.arr[i.row()]
        except:
            self.product.splits.selected=None
            
    def on_tblEPS_customContextMenuRequested(self,  pos):
        if self.selEstimationEPS==None:
            self.actionEstimationEPSDelete.setEnabled(False)
        else:
            self.actionEstimationEPSDelete.setEnabled(True)
        menu=QMenu()
        menu.addAction(self.actionEstimationEPSNew)
        menu.addAction(self.actionEstimationEPSDelete)    
        menu.exec_(self.tblEPS.mapToGlobal(pos))
            
    def on_tblDPSPaid_itemSelectionChanged(self):
        self.selDPS=None
        try:
            for i in self.tblDPSPaid.selectedItems():#itera por cada item no row.        
                if i.column()==0:
                    self.selDPS=self.product.dps.arr[i.row()]
        except:
            self.selDPS=None
        
            
    def on_tblDPSPaid_customContextMenuRequested(self,  pos):
        if self.selDPS==None:
            self.actionDPSDelete.setEnabled(False)
            self.actionDividendXuNew.setEnabled(False)
        else:
            self.actionDPSDelete.setEnabled(True)
            self.actionDividendXuNew.setEnabled(True)
            
        menu=QMenu()
        menu.addAction(self.actionDPSNew)
        menu.addAction(self.actionDPSDelete)    
        menu.addSeparator()
        menu.addAction(self.actionDPSImport)
        if self.investment!=None:
            menu.addSeparator()
            menu.addAction(self.actionDividendXuNew)
        menu.exec_(self.tblDPSPaid.mapToGlobal(pos))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgProducts.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgProducts(object):
    def setupUi(self, wdgProducts):
        wdgProducts.setObjectName("wdgProducts")
        wdgProducts.resize(892, 654)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(wdgProducts)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.lbl = QtWidgets.QLabel(wdgProducts)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout_3.addWidget(self.lbl)
        self.groupBox = QtWidgets.QGroupBox(wdgProducts)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.txt = QtWidgets.QLineEdit(self.groupBox)
        self.txt.setObjectName("txt")
        self.horizontalLayout.addWidget(self.txt)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(self.groupBox)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.chkStockExchange = QtWidgets.QCheckBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.chkStockExchange.sizePolicy().hasHeightForWidth())
        self.chkStockExchange.setSizePolicy(sizePolicy)
        self.chkStockExchange.setObjectName("chkStockExchange")
        self.horizontalLayout_2.addWidget(self.chkStockExchange)
        self.cmbStockExchange = QtWidgets.QComboBox(self.groupBox)
        self.cmbStockExchange.setEnabled(False)
        self.cmbStockExchange.setObjectName("cmbStockExchange")
        self.horizontalLayout_2.addWidget(self.cmbStockExchange)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.cmd = QtWidgets.QToolButton(self.groupBox)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmd.setIcon(icon)
        self.cmd.setObjectName("cmd")
        self.horizontalLayout_3.addWidget(self.cmd)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setEnabled(False)
        self.label_2.setText("")
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.tblInvestments = myQTableWidget(wdgProducts)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tblInvestments.sizePolicy().hasHeightForWidth())
        self.tblInvestments.setSizePolicy(sizePolicy)
        self.tblInvestments.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestments.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblInvestments.setAlternatingRowColors(True)
        self.tblInvestments.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblInvestments.setObjectName("tblInvestments")
        self.tblInvestments.setColumnCount(8)
        self.tblInvestments.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(7, item)
        self.tblInvestments.horizontalHeader().setStretchLastSection(False)
        self.tblInvestments.verticalHeader().setVisible(False)
        self.verticalLayout_3.addWidget(self.tblInvestments)
        self.lblFound = QtWidgets.QLabel(wdgProducts)
        self.lblFound.setObjectName("lblFound")
        self.verticalLayout_3.addWidget(self.lblFound)
        self.horizontalLayout_4.addLayout(self.verticalLayout_3)
        self.actionProductReport = QtWidgets.QAction(wdgProducts)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/books.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionProductReport.setIcon(icon1)
        self.actionProductReport.setObjectName("actionProductReport")
        self.actionSortTPCDiario = QtWidgets.QAction(wdgProducts)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/document-preview.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSortTPCDiario.setIcon(icon2)
        self.actionSortTPCDiario.setObjectName("actionSortTPCDiario")
        self.actionSortTPCAnual = QtWidgets.QAction(wdgProducts)
        self.actionSortTPCAnual.setIcon(icon2)
        self.actionSortTPCAnual.setObjectName("actionSortTPCAnual")
        self.actionSortName = QtWidgets.QAction(wdgProducts)
        self.actionSortName.setIcon(icon2)
        self.actionSortName.setObjectName("actionSortName")
        self.actionSortDividend = QtWidgets.QAction(wdgProducts)
        self.actionSortDividend.setIcon(icon2)
        self.actionSortDividend.setObjectName("actionSortDividend")
        self.actionSortHour = QtWidgets.QAction(wdgProducts)
        self.actionSortHour.setIcon(icon2)
        self.actionSortHour.setObjectName("actionSortHour")
        self.actionIbex35 = QtWidgets.QAction(wdgProducts)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/countries/spain.gif"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionIbex35.setIcon(icon3)
        self.actionIbex35.setObjectName("actionIbex35")
        self.actionProductNew = QtWidgets.QAction(wdgProducts)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionProductNew.setIcon(icon4)
        self.actionProductNew.setObjectName("actionProductNew")
        self.actionProductDelete = QtWidgets.QAction(wdgProducts)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/list-remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionProductDelete.setIcon(icon5)
        self.actionProductDelete.setObjectName("actionProductDelete")
        self.actionFavorites = QtWidgets.QAction(wdgProducts)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/xulpymoney/star.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFavorites.setIcon(icon6)
        self.actionFavorites.setObjectName("actionFavorites")
        self.actionMergeCodes = QtWidgets.QAction(wdgProducts)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/xulpymoney/merge.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionMergeCodes.setIcon(icon7)
        self.actionMergeCodes.setObjectName("actionMergeCodes")
        self.actionQuoteNew = QtWidgets.QAction(wdgProducts)
        self.actionQuoteNew.setIcon(icon4)
        self.actionQuoteNew.setObjectName("actionQuoteNew")
        self.actionEstimationDPSNew = QtWidgets.QAction(wdgProducts)
        self.actionEstimationDPSNew.setIcon(icon4)
        self.actionEstimationDPSNew.setObjectName("actionEstimationDPSNew")
        self.actionPurge = QtWidgets.QAction(wdgProducts)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/xulpymoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPurge.setIcon(icon8)
        self.actionPurge.setObjectName("actionPurge")
        self.actionPurchaseGraphic = QtWidgets.QAction(wdgProducts)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/xulpymoney/report.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPurchaseGraphic.setIcon(icon9)
        self.actionPurchaseGraphic.setObjectName("actionPurchaseGraphic")
        self.actionProductPriceLastRemove = QtWidgets.QAction(wdgProducts)
        self.actionProductPriceLastRemove.setIcon(icon5)
        self.actionProductPriceLastRemove.setObjectName("actionProductPriceLastRemove")

        self.retranslateUi(wdgProducts)
        self.chkStockExchange.toggled['bool'].connect(self.cmbStockExchange.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(wdgProducts)
        wdgProducts.setTabOrder(self.txt, self.tblInvestments)

    def retranslateUi(self, wdgProducts):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgProducts", "Products list"))
        self.groupBox.setTitle(_translate("wdgProducts", "Select your search"))
        self.label.setText(_translate("wdgProducts", "Search by code, ISIN, ticker or product name"))
        self.chkStockExchange.setText(_translate("wdgProducts", "Filter by stock exchange"))
        item = self.tblInvestments.horizontalHeaderItem(0)
        item.setText(_translate("wdgProducts", "Code"))
        item = self.tblInvestments.horizontalHeaderItem(1)
        item.setText(_translate("wdgProducts", "Product"))
        item = self.tblInvestments.horizontalHeaderItem(2)
        item.setText(_translate("wdgProducts", "ISIN"))
        item = self.tblInvestments.horizontalHeaderItem(3)
        item.setText(_translate("wdgProducts", "Last update"))
        item = self.tblInvestments.horizontalHeaderItem(4)
        item.setText(_translate("wdgProducts", "Price"))
        item = self.tblInvestments.horizontalHeaderItem(5)
        item.setText(_translate("wdgProducts", "% Daily"))
        item = self.tblInvestments.horizontalHeaderItem(6)
        item.setText(_translate("wdgProducts", "% Year to date"))
        item = self.tblInvestments.horizontalHeaderItem(7)
        item.setText(_translate("wdgProducts", "% Dividend"))
        self.lblFound.setText(_translate("wdgProducts", "Registers found"))
        self.actionProductReport.setText(_translate("wdgProducts", "Product report"))
        self.actionProductReport.setToolTip(_translate("wdgProducts", "Product report"))
        self.actionSortTPCDiario.setText(_translate("wdgProducts", "% Daily"))
        self.actionSortTPCDiario.setToolTip(_translate("wdgProducts", "% Daily"))
        self.actionSortTPCAnual.setText(_translate("wdgProducts", "% Annual"))
        self.actionSortTPCAnual.setToolTip(_translate("wdgProducts", "% Annual"))
        self.actionSortName.setText(_translate("wdgProducts", "Name"))
        self.actionSortName.setToolTip(_translate("wdgProducts", "Name"))
        self.actionSortDividend.setText(_translate("wdgProducts", "Dividend"))
        self.actionSortDividend.setToolTip(_translate("wdgProducts", "Dividend"))
        self.actionSortHour.setText(_translate("wdgProducts", "Hour"))
        self.actionSortHour.setToolTip(_translate("wdgProducts", "Hour"))
        self.actionIbex35.setText(_translate("wdgProducts", "Ibex 35"))
        self.actionProductNew.setText(_translate("wdgProducts", "New product"))
        self.actionProductNew.setToolTip(_translate("wdgProducts", "New user product"))
        self.actionProductDelete.setText(_translate("wdgProducts", "Delete product"))
        self.actionProductDelete.setToolTip(_translate("wdgProducts", "Delete user product"))
        self.actionFavorites.setText(_translate("wdgProducts", "Add to favorites"))
        self.actionFavorites.setToolTip(_translate("wdgProducts", "Add to favorites"))
        self.actionMergeCodes.setText(_translate("wdgProducts", "Merge selected codes"))
        self.actionMergeCodes.setToolTip(_translate("wdgProducts", "Merge selected codes"))
        self.actionQuoteNew.setText(_translate("wdgProducts", "New price"))
        self.actionQuoteNew.setToolTip(_translate("wdgProducts", "New price"))
        self.actionEstimationDPSNew.setText(_translate("wdgProducts", "New DPS estimation"))
        self.actionEstimationDPSNew.setToolTip(_translate("wdgProducts", "New Dividend per share estimation"))
        self.actionPurge.setText(_translate("wdgProducts", "Purge investment"))
        self.actionPurge.setToolTip(_translate("wdgProducts", "Deletes quotes innecesary. Leaves open, high, low and close quotes."))
        self.actionPurchaseGraphic.setText(_translate("wdgProducts", "Show purchase graphic"))
        self.actionPurchaseGraphic.setToolTip(_translate("wdgProducts", "Show purchase graphic"))
        self.actionProductPriceLastRemove.setText(_translate("wdgProducts", "Remove last product price"))
        self.actionProductPriceLastRemove.setToolTip(_translate("wdgProducts", "Remove last product price"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgBanks.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgBanks(object):
    def setupUi(self, wdgBanks):
        wdgBanks.setObjectName("wdgBanks")
        wdgBanks.resize(826, 591)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/bank.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        wdgBanks.setWindowIcon(icon)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(wdgBanks)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lbl = QtWidgets.QLabel(wdgBanks)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout_2.addWidget(self.lbl)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.chkActives = QtWidgets.QCheckBox(wdgBanks)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.chkActives.sizePolicy().hasHeightForWidth())
        self.chkActives.setSizePolicy(sizePolicy)
        self.chkActives.setChecked(True)
        self.chkActives.setObjectName("chkActives")
        self.verticalLayout.addWidget(self.chkActives)
        self.line = QtWidgets.QFrame(wdgBanks)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.groupBox_3 = QtWidgets.QGroupBox(wdgBanks)
        self.groupBox_3.setObjectName("groupBox_3")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBox_3)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.tblEB = myQTableWidget(self.groupBox_3)
        self.tblEB.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblEB.setAlternatingRowColors(True)
        self.tblEB.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblEB.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblEB.setObjectName("tblEB")
        self.tblEB.setColumnCount(3)
        self.tblEB.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblEB.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblEB.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblEB.setHorizontalHeaderItem(2, item)
        self.tblEB.verticalHeader().setVisible(False)
        self.horizontalLayout_4.addWidget(self.tblEB)
        self.verticalLayout.addWidget(self.groupBox_3)
        self.horizontalLayout_5.addLayout(self.verticalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.groupBox_2 = QtWidgets.QGroupBox(wdgBanks)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.tblAccounts = myQTableWidget(self.groupBox_2)
        self.tblAccounts.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblAccounts.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblAccounts.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblAccounts.setObjectName("tblAccounts")
        self.tblAccounts.setColumnCount(3)
        self.tblAccounts.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblAccounts.setHorizontalHeaderItem(2, item)
        self.tblAccounts.verticalHeader().setVisible(False)
        self.horizontalLayout_2.addWidget(self.tblAccounts)
        self.horizontalLayout_3.addWidget(self.groupBox_2)
        self.groupBox = QtWidgets.QGroupBox(wdgBanks)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tblInvestments = myQTableWidget(self.groupBox)
        self.tblInvestments.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestments.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblInvestments.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tblInvestments.setObjectName("tblInvestments")
        self.tblInvestments.setColumnCount(3)
        self.tblInvestments.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestments.setHorizontalHeaderItem(2, item)
        self.tblInvestments.verticalHeader().setVisible(False)
        self.horizontalLayout.addWidget(self.tblInvestments)
        self.horizontalLayout_3.addWidget(self.groupBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        self.actionInvestmentReport = QtWidgets.QAction(wdgBanks)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/report2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionInvestmentReport.setIcon(icon1)
        self.actionInvestmentReport.setObjectName("actionInvestmentReport")
        self.actionAccountReport = QtWidgets.QAction(wdgBanks)
        self.actionAccountReport.setIcon(icon1)
        self.actionAccountReport.setObjectName("actionAccountReport")
        self.actionBankAdd = QtWidgets.QAction(wdgBanks)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionBankAdd.setIcon(icon2)
        self.actionBankAdd.setObjectName("actionBankAdd")
        self.actionBankDelete = QtWidgets.QAction(wdgBanks)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/button_cancel.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionBankDelete.setIcon(icon3)
        self.actionBankDelete.setObjectName("actionBankDelete")
        self.actionBankEdit = QtWidgets.QAction(wdgBanks)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/editar.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionBankEdit.setIcon(icon4)
        self.actionBankEdit.setObjectName("actionBankEdit")
        self.actionActive = QtWidgets.QAction(wdgBanks)
        self.actionActive.setCheckable(True)
        self.actionActive.setObjectName("actionActive")

        self.retranslateUi(wdgBanks)
        QtCore.QMetaObject.connectSlotsByName(wdgBanks)

    def retranslateUi(self, wdgBanks):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgBanks", "Banks list"))
        self.chkActives.setText(_translate("wdgBanks", "Show only active entities"))
        self.groupBox_3.setTitle(_translate("wdgBanks", "Banks"))
        item = self.tblEB.horizontalHeaderItem(0)
        item.setText(_translate("wdgBanks", "Banks"))
        item = self.tblEB.horizontalHeaderItem(1)
        item.setText(_translate("wdgBanks", "Active"))
        item = self.tblEB.horizontalHeaderItem(2)
        item.setText(_translate("wdgBanks", "Balance"))
        self.groupBox_2.setTitle(_translate("wdgBanks", "Accounts"))
        item = self.tblAccounts.horizontalHeaderItem(0)
        item.setText(_translate("wdgBanks", "Account"))
        item = self.tblAccounts.horizontalHeaderItem(1)
        item.setText(_translate("wdgBanks", "Active"))
        item = self.tblAccounts.horizontalHeaderItem(2)
        item.setText(_translate("wdgBanks", "Balance"))
        self.groupBox.setTitle(_translate("wdgBanks", "Investments"))
        item = self.tblInvestments.horizontalHeaderItem(0)
        item.setText(_translate("wdgBanks", "Investment"))
        item = self.tblInvestments.horizontalHeaderItem(1)
        item.setText(_translate("wdgBanks", "Active"))
        item = self.tblInvestments.horizontalHeaderItem(2)
        item.setText(_translate("wdgBanks", "Balance"))
        self.actionInvestmentReport.setText(_translate("wdgBanks", "Investment report"))
        self.actionInvestmentReport.setToolTip(_translate("wdgBanks", "Investment report"))
        self.actionAccountReport.setText(_translate("wdgBanks", "Account report"))
        self.actionAccountReport.setToolTip(_translate("wdgBanks", "Account report"))
        self.actionBankAdd.setText(_translate("wdgBanks", "New bank"))
        self.actionBankAdd.setToolTip(_translate("wdgBanks", "New bank"))
        self.actionBankDelete.setText(_translate("wdgBanks", "Delete bank"))
        self.actionBankDelete.setToolTip(_translate("wdgBanks", "Delete bank"))
        self.actionBankEdit.setText(_translate("wdgBanks", "Edit bank"))
        self.actionBankEdit.setToolTip(_translate("wdgBanks", "Edit bank"))
        self.actionActive.setText(_translate("wdgBanks", "Is it active?"))
        self.actionActive.setToolTip(_translate("wdgBanks", "Is it active?"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



from xulpymoney.libxulpymoney import Percentage, Quote,  ProductUpdate, Assets
from xulpymoney.libxulpymoneyfunctions import days2string, qmessagebox
from PyQt5.QtCore import Qt,  pyqtSlot
from PyQt5.QtWidgets import QMenu, QWidget
from xulpymoney.ui.Ui_wdgInvestments import Ui_wdgInvestments
from xulpymoney.ui.frmInvestmentReport import frmInvestmentReport
from xulpymoney.ui.frmQuotesIBM import frmQuotesIBM
from xulpymoney.ui.frmProductReport import frmProductReport

class wdgInvestments(QWidget, Ui_wdgInvestments):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.investments=None
        self.selInvestment=None##Apunta a un objeto inversión
        self.loadedinactive=False

        self.tblInvestments.settings(self.mem, "wdgInvestments")
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
    
    def tblInvestments_reload(self):
        """Función que carga la tabla de inversiones con el orden que tenga el arr serl.investments"""
        if self.chkInactivas.checkState()==Qt.Checked:
            self.tblInvestments.setSaveSettings(False) #Must be before than hidden, because hide resizes
            self.tblInvestments.setColumnHidden(8, True)
        else:
            self.tblInvestments.setSaveSettings(True)
            self.tblInvestments.setColumnHidden(8, False)

        self.investments.myqtablewidget(self.tblInvestments)
        self.lblTotal_update()
    
    ## Updates lblTotal
    def lblTotal_update(self):
        invested=self.investments.invested()
        pendiente=self.investments.pendiente()
        if invested.isZero():
            self.lblTotal.setText(self.tr("There aren't invested assets"))
        else:
            self.lblTotal.setText(self.tr("Invested assets: {0}. Pending: {1}{2} = {3} ({4} assets)\nDaily Diff: {5}. Investment average age: {6}").format(
                            invested,self.investments.pendiente_positivo(),
                            self.investments.pendiente_negativo(),  
                            pendiente, 
                            Percentage(pendiente, invested),  
                            self.investments.gains_last_day()
                            , days2string(self.investments.average_age())))


    @pyqtSlot() 
    def on_actionActive_triggered(self):
        if self.selInvestment.account.eb.qmessagebox_inactive()  or self.selInvestment.account.qmessagebox_inactive():
            return  
    
        self.selInvestment.active=not self.selInvestment.active
        self.selInvestment.save()
        self.mem.con.commit()     

        self.chkInactivas.setCheckState(Qt.Unchecked)
        self.on_chkInactivas_stateChanged(Qt.Unchecked)

            
    @pyqtSlot() 
    def on_actionInvestmentDelete_triggered(self):
        """
            on_tblInvestments_customContextMenuRequested validates if it's deletable
        """
        self.selInvestment.borrar()
        self.mem.data.investments.remove(self.selInvestment)
        self.mem.con.commit()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

          
    @pyqtSlot() 
    def on_actionInvestmentAdd_triggered(self):
        w=frmInvestmentReport(self.mem,   None, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
        
    @pyqtSlot() 
    def on_actionInvestmentReport_triggered(self):
        w=frmInvestmentReport(self.mem,  self.selInvestment, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

    @pyqtSlot() 
    def on_actionProduct_triggered(self):
        w=frmProductReport(self.mem, self.selInvestment.product, self.selInvestment, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

    @pyqtSlot() 
    def on_actionProductUpdate_triggered(self):
        update=ProductUpdate(self.mem)
        update.setCommands(self.selInvestment.product)
        quotes=update.run()
        quotes.save()
        self.mem.con.commit()
        quotes.print()
        
    @pyqtSlot() 
    def on_actionSameProduct_triggered(self):
        inv=self.mem.data.investments.investment_merging_current_operations_with_same_product(self.selInvestment.product)
        w=frmInvestmentReport(self.mem, inv, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
                    
    @pyqtSlot() 
    def on_actionSameProductFIFO_triggered(self):
        inv=self.mem.data.investments.investment_merging_operations_with_same_product(self.selInvestment.product)
        w=frmInvestmentReport(self.mem, inv, self)
        w.exec_()
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
            
            
    @pyqtSlot() 
    def on_actionProductPrice_triggered(self):
        w=frmQuotesIBM(self.mem, self.selInvestment.product,None,  self)
        w.exec_()
        self.selInvestment.product.needStatus(1, downgrade_to=0)
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())
        
    @pyqtSlot() 
    def on_actionProductPriceLastRemove_triggered(self):
        self.selInvestment.product.result.basic.last.delete()
        self.mem.con.commit()
        self.selInvestment.product.needStatus(1, downgrade_to=0)
        self.on_chkInactivas_stateChanged(self.chkInactivas.checkState())

    @pyqtSlot() 
    def on_actionSortTPCDiario_triggered(self):
        if self.investments.order_by_percentage_daily():
            self.tblInvestments_reload()    
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))  
            self.tblInvestments_reload()       
        
    @pyqtSlot() 
    def on_actionSortTPCVenta_triggered(self):
        if self.investments.order_by_percentage_invested() and self.investments.order_by_percentage_sellingpoint():
            self.tblInvestments_reload()    
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))     
            self.tblInvestments_reload()    
        
    @pyqtSlot() 
    def on_actionSortTPC_triggered(self):
        if self.investments.order_by_percentage_invested():
            self.tblInvestments_reload()    
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))     
            self.tblInvestments_reload()    
        
    @pyqtSlot() 
    def on_actionSortHour_triggered(self):
        if self.investments.order_by_datetime():
            self.tblInvestments_reload()    
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))     
            self.tblInvestments_reload()    
        
    @pyqtSlot() 
    def on_actionSortName_triggered(self):
        if self.investments.order_by_name():
            self.tblInvestments_reload()    
        else:
            qmessagebox(self.tr("I couldn't order data due to they have null values"))     
            self.tblInvestments_reload()    
            
    @pyqtSlot(int) 
    def on_chkInactivas_stateChanged(self, state):
        if state==Qt.Unchecked:
            self.investments=self.mem.data.investments_active()
            self.on_actionSortTPCVenta_triggered()
        else:
            self.investments=self.mem.data.investments_inactive()
            self.on_actionSortName_triggered()
        self.tblInvestments.clearSelection()
        self.selInvestment=None   

    def on_tblInvestments_customContextMenuRequested(self,  pos):
        if self.selInvestment==None:
            self.actionInvestmentReport.setEnabled(False)
            self.actionInvestmentDelete.setEnabled(False)
            self.actionActive.setEnabled(False)
            self.actionProduct.setEnabled(False)
            self.actionProductPrice.setEnabled(False)
            self.actionProductPriceLastRemove.setEnabled(False)
            self.actionProductUpdate.setEnabled(False)
        else:
            self.actionInvestmentReport.setEnabled(True)
            self.actionActive.setEnabled(True)       
            self.actionProduct.setEnabled(True)
            if self.mem.data.investments.numberWithSameProduct(self.selInvestment.product)>1:
                self.actionSameProduct.setEnabled(True)
                self.actionSameProductFIFO.setEnabled(True)
            else:
                self.actionSameProduct.setEnabled(False)
                self.actionSameProductFIFO.setEnabled(False)
            self.actionProductPrice.setEnabled(True)
            if self.selInvestment.is_deletable()==True:
                self.actionInvestmentDelete.setEnabled(True)
            else:
                self.actionInvestmentDelete.setEnabled(False)
                
            self.actionProductPriceLastRemove.setEnabled(True)
            self.actionProductUpdate.setEnabled(True)
                
            if self.selInvestment.active==True:
                self.actionActive.setText(self.tr('Deactivate investment'))
            else:
                self.actionActive.setText(self.tr('Activate investment'))
                
        menu=QMenu()
        menu.addAction(self.actionInvestmentAdd)
        menu.addAction(self.actionInvestmentDelete)   
        menu.addSeparator()   
        menu.addAction(self.actionInvestmentReport)     
        menu.addAction(self.actionProduct)
        menu.addSeparator()
        menu.addAction(self.actionSameProduct)
        menu.addAction(self.actionSameProductFIFO)   
        menu.addSeparator()
        menu.addAction(self.actionProductPrice)
        menu.addAction(self.actionProductPriceLastRemove)
        menu.addAction(self.actionProductUpdate)
        menu.addSeparator()
        menu.addAction(self.actionActive)
        menu.addSeparator()        
        ordenar=QMenu(self.tr("Order by"))
        ordenar.addAction(self.actionSortName)
        ordenar.addAction(self.actionSortHour)
        ordenar.addAction(self.actionSortTPCDiario)
        ordenar.addAction(self.actionSortTPC)
        ordenar.addAction(self.actionSortTPCVenta)
        menu.addMenu(ordenar)        
        menu.exec_(self.tblInvestments.mapToGlobal(pos))

    def on_tblInvestments_itemSelectionChanged(self):
        self.selInvestment=None
        for i in self.tblInvestments.selectedItems():#itera por cada item no row.
            self.selInvestment=self.investments.arr[i.row()]

    @pyqtSlot(int, int) 
    def on_tblInvestments_cellDoubleClicked(self, row, column):
        if column==8:#TPC Venta
            qmessagebox(self.tr("Shares number: {0}").format(self.selInvestment.shares())+"\n"+
                    self.tr("Purchase price average: {0}").format(self.selInvestment.op_actual.average_price().local())+"\n"+
                    self.tr("Selling point: {}").format(self.selInvestment.venta)+"\n"+
                    self.tr("Selling all shares you get {}").format(self.selInvestment.op_actual.pendiente(Quote(self.mem).init__create(self.selInvestment.product, self.mem.localzone.now(),  self.selInvestment.venta)).local())
            )  
        else:
            self.on_actionInvestmentReport_triggered()

## Class to show Zero Risk investments
class wdgInvestmentsZeroRisk(wdgInvestments):
    def __init__(self, mem, parent=None):
        wdgInvestments.__init__(self, mem, parent)
        self.lbl.setText(self.tr("Zero risk investment list"))    

    ## Updates lblTotal, overrides supper method
    def lblTotal_update(self):
        investments=self.investments.balance()
        accounts=self.mem.data.accounts_active().balance()
        total=Assets(self.mem).saldo_total(self.mem.data.investments_active(), self.mem.localzone.now())
        if self.chkInactivas.checkState()==Qt.Checked:
            self.lblTotal.setText(self.tr("There aren't invested assets"))
        else:
            self.lblTotal.setText(self.tr("""Accounts balance: {0}
Zero risk investments balance: {1}
Zero risk assests balance: {2} ( {3} from your total assets {4} )""").format(
                            accounts, 
                            investments, 
                            accounts+investments, 
                            Percentage((accounts+investments).amount, total.amount), 
                            total
                        ))

    ## Overrides supper method and selects investments with zero risk
    @pyqtSlot(int) 
    def on_chkInactivas_stateChanged(self, state):
        if state==Qt.Unchecked:
            self.investments=self.mem.data.investments_active().InvestmentManager_with_investments_with_zero_risk()
            self.on_actionSortTPCVenta_triggered()
        else:
            self.investments=self.mem.data.investments_inactive().InvestmentManager_with_investments_with_zero_risk()
            self.on_actionSortName_triggered()
        self.tblInvestments.clearSelection()
        self.selInvestment=None   



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgCalculator.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgCalculator(object):
    def setupUi(self, wdgCalculator):
        wdgCalculator.setObjectName("wdgCalculator")
        wdgCalculator.resize(851, 852)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(wdgCalculator)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(wdgCalculator)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.label_7 = QtWidgets.QLabel(wdgCalculator)
        self.label_7.setMaximumSize(QtCore.QSize(64, 64))
        self.label_7.setText("")
        self.label_7.setPixmap(QtGui.QPixmap(":/xulpymoney/kcalc.png"))
        self.label_7.setScaledContents(True)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_4.addWidget(self.label_7)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.groupBox = QtWidgets.QGroupBox(wdgCalculator)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.cmbProducts = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbProducts.sizePolicy().hasHeightForWidth())
        self.cmbProducts.setSizePolicy(sizePolicy)
        self.cmbProducts.setObjectName("cmbProducts")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.cmbProducts)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.cmbPrice = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbPrice.sizePolicy().hasHeightForWidth())
        self.cmbPrice.setSizePolicy(sizePolicy)
        self.cmbPrice.setObjectName("cmbPrice")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.cmbPrice)
        self.txtProductPrice = myQLineEdit(self.groupBox)
        self.txtProductPrice.setEnabled(True)
        self.txtProductPrice.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtProductPrice.setReadOnly(True)
        self.txtProductPrice.setObjectName("txtProductPrice")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtProductPrice)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.spnProductPriceVariation = QtWidgets.QDoubleSpinBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spnProductPriceVariation.sizePolicy().hasHeightForWidth())
        self.spnProductPriceVariation.setSizePolicy(sizePolicy)
        self.spnProductPriceVariation.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spnProductPriceVariation.setMinimum(-99.99)
        self.spnProductPriceVariation.setObjectName("spnProductPriceVariation")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.spnProductPriceVariation)
        self.lblLeveraged = QtWidgets.QLabel(self.groupBox)
        self.lblLeveraged.setObjectName("lblLeveraged")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.lblLeveraged)
        self.txtLeveraged = myQLineEdit(self.groupBox)
        self.txtLeveraged.setEnabled(True)
        self.txtLeveraged.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtLeveraged.setReadOnly(True)
        self.txtLeveraged.setObjectName("txtLeveraged")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.txtLeveraged)
        self.lblFinalPrice = QtWidgets.QLabel(self.groupBox)
        self.lblFinalPrice.setObjectName("lblFinalPrice")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.lblFinalPrice)
        self.txtFinalPrice = myQLineEdit(self.groupBox)
        self.txtFinalPrice.setEnabled(True)
        self.txtFinalPrice.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtFinalPrice.setObjectName("txtFinalPrice")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.txtFinalPrice)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.cmbInvestments = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmbInvestments.sizePolicy().hasHeightForWidth())
        self.cmbInvestments.setSizePolicy(sizePolicy)
        self.cmbInvestments.setObjectName("cmbInvestments")
        self.horizontalLayout_6.addWidget(self.cmbInvestments)
        self.chkWithoutShares = QtWidgets.QCheckBox(self.groupBox)
        self.chkWithoutShares.setObjectName("chkWithoutShares")
        self.horizontalLayout_6.addWidget(self.chkWithoutShares)
        self.formLayout_2.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_6)
        self.horizontalLayout_3.addLayout(self.formLayout_2)
        self.verticalLayout.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(wdgCalculator)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.lblAmount = QtWidgets.QLabel(self.groupBox_2)
        self.lblAmount.setObjectName("lblAmount")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lblAmount)
        self.txtAmount = myQLineEdit(self.groupBox_2)
        self.txtAmount.setEnabled(True)
        self.txtAmount.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtAmount.setObjectName("txtAmount")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.txtAmount)
        self.lblShares = QtWidgets.QLabel(self.groupBox_2)
        self.lblShares.setObjectName("lblShares")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lblShares)
        self.txtShares = myQLineEdit(self.groupBox_2)
        self.txtShares.setEnabled(True)
        self.txtShares.setText("")
        self.txtShares.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtShares.setReadOnly(True)
        self.txtShares.setObjectName("txtShares")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtShares)
        self.horizontalLayout_2.addLayout(self.formLayout)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.cmdGraph = QtWidgets.QPushButton(wdgCalculator)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/report.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdGraph.setIcon(icon)
        self.cmdGraph.setObjectName("cmdGraph")
        self.horizontalLayout_5.addWidget(self.cmdGraph)
        self.line = QtWidgets.QFrame(wdgCalculator)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_5.addWidget(self.line)
        self.cmdOrder = QtWidgets.QPushButton(wdgCalculator)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdOrder.setIcon(icon1)
        self.cmdOrder.setObjectName("cmdOrder")
        self.horizontalLayout_5.addWidget(self.cmdOrder)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.groupBox_3 = QtWidgets.QGroupBox(wdgCalculator)
        self.groupBox_3.setObjectName("groupBox_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox_3)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.table = myQTableWidget(self.groupBox_3)
        self.table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.table.setAlternatingRowColors(True)
        self.table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.table.setObjectName("table")
        self.table.setColumnCount(3)
        self.table.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(2, item)
        self.table.verticalHeader().setVisible(False)
        self.horizontalLayout.addWidget(self.table)
        self.verticalLayout.addWidget(self.groupBox_3)
        self.horizontalLayout_8.addLayout(self.verticalLayout)

        self.retranslateUi(wdgCalculator)
        QtCore.QMetaObject.connectSlotsByName(wdgCalculator)

    def retranslateUi(self, wdgCalculator):
        _translate = QtCore.QCoreApplication.translate
        self.lbl.setText(_translate("wdgCalculator", "Investment calculator"))
        self.groupBox.setTitle(_translate("wdgCalculator", "Product settings"))
        self.label_3.setText(_translate("wdgCalculator", "Select a product"))
        self.label.setText(_translate("wdgCalculator", "Select your personal investment"))
        self.label_2.setText(_translate("wdgCalculator", "Variation from current price"))
        self.spnProductPriceVariation.setSuffix(_translate("wdgCalculator", " %"))
        self.lblLeveraged.setText(_translate("wdgCalculator", "Leveraged product"))
        self.txtLeveraged.setText(_translate("wdgCalculator", "1"))
        self.lblFinalPrice.setText(_translate("wdgCalculator", "Final price"))
        self.chkWithoutShares.setText(_translate("wdgCalculator", "Without shares"))
        self.groupBox_2.setTitle(_translate("wdgCalculator", "Shares calc"))
        self.lblAmount.setText(_translate("wdgCalculator", "Amount to invest"))
        self.lblShares.setText(_translate("wdgCalculator", "Shares"))
        self.cmdGraph.setText(_translate("wdgCalculator", "Show purchase graph"))
        self.cmdOrder.setText(_translate("wdgCalculator", "Add order annotation"))
        self.groupBox_3.setTitle(_translate("wdgCalculator", "Calc estimations"))
        item = self.table.horizontalHeaderItem(0)
        item.setText(_translate("wdgCalculator", "% Gains"))
        item = self.table.horizontalHeaderItem(1)
        item.setText(_translate("wdgCalculator", "Price"))
        item = self.table.horizontalHeaderItem(2)
        item.setText(_translate("wdgCalculator", "Gains"))


from xulpymoney.ui.myqlineedit import myQLineEdit
from xulpymoney.ui.myqtablewidget import myQTableWidget
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/frmSharesTransfer.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_frmSharesTransfer(object):
    def setupUi(self, frmSharesTransfer):
        frmSharesTransfer.setObjectName("frmSharesTransfer")
        frmSharesTransfer.resize(559, 167)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/transfer.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        frmSharesTransfer.setWindowIcon(icon)
        frmSharesTransfer.setSizeGripEnabled(True)
        self.horizontalLayout = QtWidgets.QHBoxLayout(frmSharesTransfer)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lbl = QtWidgets.QLabel(frmSharesTransfer)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lbl.setFont(font)
        self.lbl.setStyleSheet("color: rgb(0, 128, 0);")
        self.lbl.setAlignment(QtCore.Qt.AlignCenter)
        self.lbl.setObjectName("lbl")
        self.verticalLayout.addWidget(self.lbl)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.ExpandingFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.combo = QtWidgets.QComboBox(frmSharesTransfer)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.combo.sizePolicy().hasHeightForWidth())
        self.combo.setSizePolicy(sizePolicy)
        self.combo.setObjectName("combo")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.combo)
        self.lblCombo = QtWidgets.QLabel(frmSharesTransfer)
        self.lblCombo.setObjectName("lblCombo")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lblCombo)
        self.lblNumero = QtWidgets.QLabel(frmSharesTransfer)
        self.lblNumero.setObjectName("lblNumero")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lblNumero)
        self.txtAcciones = myQLineEdit(frmSharesTransfer)
        self.txtAcciones.setEnabled(False)
        self.txtAcciones.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtAcciones.setObjectName("txtAcciones")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txtAcciones)
        self.lblComisionLabel = QtWidgets.QLabel(frmSharesTransfer)
        self.lblComisionLabel.setObjectName("lblComisionLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.lblComisionLabel)
        self.txtComision = myQLineEdit(frmSharesTransfer)
        self.txtComision.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.txtComision.setObjectName("txtComision")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txtComision)
        self.verticalLayout.addLayout(self.formLayout)
        self.buttons = QtWidgets.QDialogButtonBox(frmSharesTransfer)
        self.buttons.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttons.setObjectName("buttons")
        self.verticalLayout.addWidget(self.buttons, 0, QtCore.Qt.AlignHCenter)
        self.horizontalLayout.addLayout(self.verticalLayout)

        self.retranslateUi(frmSharesTransfer)
        QtCore.QMetaObject.connectSlotsByName(frmSharesTransfer)

    def retranslateUi(self, frmSharesTransfer):
        _translate = QtCore.QCoreApplication.translate
        frmSharesTransfer.setWindowTitle(_translate("frmSharesTransfer", "Shares transfer"))
        self.lblCombo.setText(_translate("frmSharesTransfer", "Select the destiny for the shares"))
        self.lblNumero.setText(_translate("frmSharesTransfer", "Number of shares"))
        self.lblComisionLabel.setText(_translate("frmSharesTransfer", "Shares transfer comission"))
        self.txtComision.setText(_translate("frmSharesTransfer", "0"))


from xulpymoney.ui.myqlineedit import myQLineEdit
import xulpymoney.images.xulpymoney_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgSimulationsAdd.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgSimulationsAdd(object):
    def setupUi(self, wdgSimulationsAdd):
        wdgSimulationsAdd.setObjectName("wdgSimulationsAdd")
        wdgSimulationsAdd.resize(782, 573)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(wdgSimulationsAdd)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblTitulo = QtWidgets.QLabel(wdgSimulationsAdd)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout.addWidget(self.lblTitulo)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.lblPixmap = QtWidgets.QLabel(wdgSimulationsAdd)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblPixmap.sizePolicy().hasHeightForWidth())
        self.lblPixmap.setSizePolicy(sizePolicy)
        self.lblPixmap.setMinimumSize(QtCore.QSize(48, 48))
        self.lblPixmap.setMaximumSize(QtCore.QSize(48, 48))
        self.lblPixmap.setPixmap(QtGui.QPixmap(":/xulpymoney/replication.png"))
        self.lblPixmap.setScaledContents(True)
        self.lblPixmap.setAlignment(QtCore.Qt.AlignCenter)
        self.lblPixmap.setObjectName("lblPixmap")
        self.horizontalLayout_4.addWidget(self.lblPixmap)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.cmbSimulationTypes = QtWidgets.QComboBox(wdgSimulationsAdd)
        self.cmbSimulationTypes.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.cmbSimulationTypes.setObjectName("cmbSimulationTypes")
        self.verticalLayout_2.addWidget(self.cmbSimulationTypes)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(wdgSimulationsAdd)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.wdgStarting = wdgDatetime(self.groupBox)
        self.wdgStarting.setObjectName("wdgStarting")
        self.horizontalLayout.addWidget(self.wdgStarting)
        self.horizontalLayout_3.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(wdgSimulationsAdd)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.wdgEnding = wdgDatetime(self.groupBox_2)
        self.wdgEnding.setObjectName("wdgEnding")
        self.horizontalLayout_2.addWidget(self.wdgEnding)
        self.horizontalLayout_3.addWidget(self.groupBox_2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.buttonbox = QtWidgets.QDialogButtonBox(wdgSimulationsAdd)
        self.buttonbox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonbox.setObjectName("buttonbox")
        self.verticalLayout_2.addWidget(self.buttonbox)
        self.horizontalLayout_5.addLayout(self.verticalLayout_2)

        self.retranslateUi(wdgSimulationsAdd)
        QtCore.QMetaObject.connectSlotsByName(wdgSimulationsAdd)

    def retranslateUi(self, wdgSimulationsAdd):
        _translate = QtCore.QCoreApplication.translate
        wdgSimulationsAdd.setWindowTitle(_translate("wdgSimulationsAdd", "Form"))
        self.lblTitulo.setText(_translate("wdgSimulationsAdd", "Create new simulation"))
        self.groupBox.setTitle(_translate("wdgSimulationsAdd", "Start date and time"))
        self.groupBox_2.setTitle(_translate("wdgSimulationsAdd", "End date and time"))


from xulpymoney.ui.wdgDatetime import wdgDatetime
import xulpymoney.images.xulpymoney_rc



from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QDialog
from xulpymoney.ui.Ui_frmAccountOperationsAdd import Ui_frmAccountOperationsAdd
from xulpymoney.libxulpymoney import  AccountOperation,  Comment, CreditCardOperation
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.libxulpymoneytypes import eComment
from datetime import timedelta

class frmAccountOperationsAdd(QDialog, Ui_frmAccountOperationsAdd):
    AccountOperationChanged=pyqtSignal(AccountOperation)
    CreditCardOperationChanged=pyqtSignal(CreditCardOperation)
    def __init__(self, mem, account=None, opercuenta=None, tarjeta=None ,  opertarjeta=None,  refund=False,  parent=None, ):
        """TIPOS DE ENTRADAS:        
         1   selAccount=x: Inserción de Opercuentas y edición de cuentas
         2   selAccount=x, opercuenta=x Modificación de opercuentas e insecioon de opertarjetas a débito
         3   selAccount=x, opercuenta=None , tarjeta=x, Inserción de opertarjetas en diferido
         4   selAccount=x, opercuenta=None , tarjeta=x, opertarjeta=x Modificación de opertarjetas
         5   selAccount=None, opercuenta=None, tarjeta=None, opertarjeta=x, refund=True Refund of opertarjetas. 
         
         Debido a que se puede cambiar de opercuenta a opercreditcard grabo el producto original, en el caso de modificación
         Original puede ser un 
            operaccount, para poder editarla
            opercreditcard, para poder editarlo y refund (aunque existe el booleano self.refund para diferenciarlo)
            -999, para productos nuevos, ya que no puedo sacar null por la señal.
            
        Luego usaré el objeto original para la modificación ya que después se sale de este dialogo y si es uno nuevo dejaré origianl a None y usarre uno nuevo que llamaré final
         """
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        
        self.refund=refund

        self.mem.conceptos.load_opercuentas_qcombobox(self.cmbConcepts)
        self.mem.data.accounts_active().qcombobox(self.cmbAccounts)
        self.mem.data.creditcards_active().qcombobox(self.cmbCreditCards)
        self.wdgDT.show_microseconds(False)

        if account==None and opercuenta==None and tarjeta==None and opertarjeta!=None and  refund==True:
            self.original=opertarjeta
            self.radCreditCards.setChecked(True)
            self.radAccounts.setEnabled(False)
            self.radCreditCards.setEnabled(False)
            self.cmbCreditCards.setEnabled(False)
            self.setWindowTitle(self.tr("Credit card operation refund"))
            self.lblTitulo.setText(self.tr("Credit card operation refund"))
            self.wdgDT.set(self.mem)
            self.cmbConcepts.setCurrentIndex(self.cmbConcepts.findData(67))
            self.cmbConcepts.setEnabled(False)
            self.cmbAccounts.setCurrentIndex(self.cmbAccounts.findData(opertarjeta.tarjeta.account.id))
            self.cmbCreditCards.setCurrentIndex(self.cmbCreditCards.findData(opertarjeta.tarjeta.id))
            self.txtImporte.setText(-opertarjeta.importe)
            self.txtComentario.setEnabled(False)
        elif opertarjeta!=None:
            self.original=opertarjeta
            self.setWindowTitle(self.tr("Credit card operation update"))
            self.lblTitulo.setText(self.tr("Credit card operation update"))
            self.radCreditCards.setChecked(True)
            self.wdgDT.set(self.mem, opertarjeta.datetime, self.mem.localzone)
            self.cmbConcepts.setCurrentIndex(self.cmbConcepts.findData(opertarjeta.concepto.id))
            self.cmbAccounts.setCurrentIndex(self.cmbAccounts.findData(opertarjeta.tarjeta.account.id))
            self.cmbCreditCards.setCurrentIndex(self.cmbCreditCards.findData(opertarjeta.tarjeta.id))
            self.txtImporte.setText(opertarjeta.importe)
            self.txtComentario.setText(opertarjeta.comentario)
        elif tarjeta!=None:
            self.original=None
            self.radCreditCards.setChecked(True)
            self.cmbCreditCards.setCurrentIndex(self.cmbCreditCards.findData(tarjeta.id))
            self.setWindowTitle(self.tr("New credit card operation"))
            self.lblTitulo.setText(self.tr("New credit card operation"))
            self.wdgDT.set(self.mem)
        elif opercuenta!=None:
            self.original=opercuenta
            self.radAccounts.setChecked(True)
            self.setWindowTitle(self.tr("Account operation update"))
            self.lblTitulo.setText(self.tr("Account operation update"))
            self.wdgDT.set(self.mem, opercuenta.datetime, self.mem.localzone)
            self.cmbConcepts.setCurrentIndex(self.cmbConcepts.findData(opercuenta.concepto.id))
            self.cmbAccounts.setCurrentIndex(self.cmbAccounts.findData(opercuenta.account.id))
            self.txtImporte.setText(opercuenta.importe)
            self.txtComentario.setText(opercuenta.comentario)
        else:
            self.original=None
            self.radAccounts.setChecked(True)
            self.setWindowTitle(self.tr("New account operation"))
            self.lblTitulo.setText(self.tr("New account operation"))
            self.wdgDT.set(self.mem)
            self.cmbAccounts.setCurrentIndex(self.cmbAccounts.findData(account.id))


    def type_and_id(self, product):
        """0 if account,  1 if credit card. Como no puedo pasar Null por la señal, saco con -999, es decir cuando sea producto nuevo"""
        if product==None:
            return [-999, -999]
        if product.__class__==AccountOperation:
            return (0, product.account.id)
        if product.__class__==CreditCardOperation:
            return (1, product.tarjeta.id)
        print (product.__class__, "ERROR",  product)
            


    def on_cmd_released(self):
        concepto=self.mem.conceptos.find_by_id(self.cmbConcepts.itemData(self.cmbConcepts.currentIndex()))
        importe=self.txtImporte.decimal()
        comentario=self.txtComentario.text()
        id_cuentas=self.cmbAccounts.itemData(self.cmbAccounts.currentIndex()) #Sólo se usará en 1 y 2.
        id_tarjetas=self.cmbCreditCards.itemData(self.cmbCreditCards.currentIndex())
        cuenta=self.mem.data.accounts_active().find_by_id(id_cuentas)
        tarjeta=self.mem.data.creditcards_active().find_by_id(id_tarjetas)
        
        if not importe:
            qmessagebox(self.tr("You must set the operation amount"))
            return        
        
        if concepto.tipooperacion.id==1 and importe>0:
            qmessagebox(self.tr("Expenses can not have a positive amount"))
            return
            
        if concepto.tipooperacion.id==2 and importe<0:
            qmessagebox(self.tr("Incomes can not have a negative amount"))
            return
            
        if self.radAccounts.isChecked():#Producto final es un operaccount
            if self.original==None:#Producto nuevo
                final=AccountOperation(self.mem)
                final.datetime=self.wdgDT.datetime()
                final.concepto=concepto
                final.tipooperacion=concepto.tipooperacion
                final.importe=importe
                final.comentario=comentario
                final.account=cuenta
                final.save()
                self.mem.con.commit()        #Se debe hacer el commit antes para que al actualizar con el signal salga todos los datos
                self.AccountOperationChanged.emit(final)
                self.wdgDT.set(self.mem, self.wdgDT.datetime()+timedelta(seconds=1), self.wdgDT.zone)
                return
            elif self.original.__class__==CreditCardOperation:#Modificación  de opercreditcard por operaccount hay que borrar opercreditcard
                final=AccountOperation(self.mem)
                final.datetime=self.wdgDT.datetime()
                final.concepto=concepto
                final.tipooperacion=concepto.tipooperacion
                final.importe=importe
                final.comentario=comentario
                final.account=cuenta
                final.save()
                self.original.borrar()
                self.mem.con.commit()        #Se debe hacer el commit antes para que al actualizar con el signal salga todos los datos
                self.AccountOperationChanged.emit(final)
                self.done(0)
                return
            elif self.original.__class__==AccountOperation:
#                origi=self.type_and_id(self.original)#Ya que se cambia en el save
                self.original.datetime=self.wdgDT.datetime()
                self.original.concepto=concepto
                self.original.tipooperacion=concepto.tipooperacion
                self.original.importe=importe
                self.original.comentario=comentario
                self.original.account=cuenta
                self.original.save()
                self.mem.con.commit()        #Se debe hacer el commit antes para que al actualizar con el signal salga todos los datos
                self.AccountOperationChanged.emit(self.original)
                self.done(0)
                return

        elif self.radCreditCards.isChecked():#Producto final es un opercreditcard
            if tarjeta.pagodiferido==False:#Pago débito
                final=AccountOperation(self.mem)
                final.datetime=self.wdgDT.datetime()
                final.concepto=concepto
                final.tipooperacion=concepto.tipooperacion
                final.importe=importe
                final.comentario=self.tr("CreditCard {0}. {1}".format(tarjeta.name, comentario))
                final.account=tarjeta.account
                final.save()
                if self.original!=None:
                    self.original.borrar()
                self.mem.con.commit()
                self.AccountOperationChanged.emit(final)
                self.done(0)
                return
            elif self.original==None:#CreditCardOperation nueva
                final=CreditCardOperation(self.mem).init__create(self.wdgDT.datetime(), concepto, concepto.tipooperacion, importe, comentario, tarjeta, False, None, None )
                final.save()
                self.mem.con.commit()
                self.CreditCardOperationChanged.emit(final)
                self.wdgDT.set(self.mem, self.wdgDT.datetime()+timedelta(seconds=1), self.wdgDT.zone)
                return
            elif self.refund==True:#Refun d         
                refund=CreditCardOperation(self.mem)
                refund.datetime=self.wdgDT.datetime()
                refund.concepto=concepto
                refund.tarjeta=tarjeta
                refund.tipooperacion=concepto.tipooperacion
                refund.pagado=False
                refund.importe=importe
                refund.comentario=Comment(self.mem).encode(eComment.CreditCardRefund, self.original)
                refund.save()
                self.mem.con.commit()        #Se debe hacer el commit antes para que al actualizar con el signal salga todos los datos
                self.CreditCardOperationChanged.emit(refund)
                self.done(0) 
                return
            elif self.original.__class__==AccountOperation:#Modificación  de opercreditcard por operaccount hay que borrar opercreditcard
                final=CreditCardOperation(self.mem).init__create(self.wdgDT.datetime(), concepto, concepto.tipooperacion, importe, comentario, tarjeta, False, None, None )
                final.save()
                self.original.borrar()
                self.mem.con.commit()        #Se debe hacer el commit antes para que al actualizar con el signal salga todos los datos
                self.CreditCardOperationChanged.emit(final)
                self.done(0)
                return
            elif self.original.__class__==CreditCardOperation:
#                origi=self.type_and_id(self.original)#Ya que se cambia en el save
                self.original.datetime=self.wdgDT.datetime()
                self.original.concepto=concepto
                self.original.tipooperacion=concepto.tipooperacion
                self.original.importe=importe
                self.original.comentario=comentario
                self.original.tarjeta=tarjeta
                self.original.save()
                self.mem.con.commit()        #Se debe hacer el commit antes para que al actualizar con el signal salga todos los datos
                self.CreditCardOperationChanged.emit(self.original)
                self.done(0)
                return

    
